"""assets"""

from string import Template

CSS = """
.thumbnail {
  position: relative;
  z-index: 0;
}

.thumbnail:hover {
  background-color: transparent;
  z-index: 50;
}

.thumbnail span {
  position: absolute;
  padding: 5px;
  left: -1000px;
  border: 1px dashed gray;
  display: none;
  color: black;
  text-decoration: none;
}

.thumbnail span > * {
  border-width: 0;
  padding: 2px;
  min-width: 800px;
}

.thumbnail:hover span {
  display: block !important;
  top: 0;
  left: 60px; /*position where enlarged image should offset horizontally */
"""


BACK_ICO = "data:image/gif;base64,R0lGODlhFAAWAMIAAP///8z//5mZmWZmZjMzMwAAAAAAAAAAACH+TlRoaXMgYXJ0IGlzIGluIHRoZSBwdWJsaWMgZG9tYWluLiBLZXZpbiBIdWdoZXMsIGtldmluaEBlaXQuY29tLCBTZXB0ZW1iZXIgMTk5NQAh+QQBAAABACwAAAAAFAAWAAADSxi63P4jEPJqEDNTu6LO3PVpnDdOFnaCkHQGBTcqRRxuWG0v+5LrNUZQ8QPqeMakkaZsFihOpyDajMCoOoJAGNVWkt7QVfzokc+LBAA7"

FOLDER_ICO = "data:image/gif;base64,R0lGODlhFAAWAMIAAP/////Mmcz//5lmMzMzMwAAAAAAAAAAACH+TlRoaXMgYXJ0IGlzIGluIHRoZSBwdWJsaWMgZG9tYWluLiBLZXZpbiBIdWdoZXMsIGtldmluaEBlaXQuY29tLCBTZXB0ZW1iZXIgMTk5NQAh+QQBAAACACwAAAAAFAAWAAADVCi63P4wyklZufjOErrvRcR9ZKYpxUB6aokGQyzHKxyO9RoTV54PPJyPBewNSUXhcWc8soJOIjTaSVJhVphWxd3CeILUbDwmgMPmtHrNIyxM8Iw7AQA7"

UNKNOWN_ICO = "data:image/gif;base64,R0lGODlhFAAWAMIAAP///8z//5mZmTMzMwAAAAAAAAAAAAAAACH+TlRoaXMgYXJ0IGlzIGluIHRoZSBwdWJsaWMgZG9tYWluLiBLZXZpbiBIdWdoZXMsIGtldmluaEBlaXQuY29tLCBTZXB0ZW1iZXIgMTk5NQAh+QQBAAABACwAAAAAFAAWAAADaDi6vPEwDECrnSO+aTvPEQcIAmGaIrhR5XmKgMq1LkoMN7ECrjDWp52r0iPpJJ0KjUAq7SxLE+sI+9V8vycFiM0iLb2O80s8JcfVJJTaGYrZYPNby5Ov6WolPD+XDJqAgSQ4EUCGQQEJADs="

HTML_TEMPLATE = f"""
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html lang="en">
    <head>
        <meta charset="utf-8" />
        <title></title>
        <style></style>
    </head>
    <body>
        <h1></h1>
        <table>
            <tr>
                <th valign="top">
                    <img src="{FOLDER_ICO}" alt="[ICO]" />
                </th>
                <th><a href="?C=N;O=D">Name</a></th>
                <th><a href="?C=M;O=A">Last modified</a></th>
                <th><a href="?C=S;O=A">Size</a></th>
                <th><a href="?C=D;O=A">Description</a></th>
            </tr>
            <tr>
                <th colspan="5"><hr /></th>
            </tr>
            <tr>
                <td valign="top">
                    <img src="{BACK_ICO}" alt="[PARENTDIR]" />
                </td>
                <td><a href="..">Parent Directory</a></td>
                <td>&nbsp;</td>
                <td align="right">-</td>
                <td>&nbsp;</td>
            </tr>
            <placeholder></placeholder>
            <tr>
                <th colspan="5"><hr /></th>
            </tr>
        </table>
        <address></address>
    </body>
</html>"""


ROW_TEMPLATE = Template(
    """<tr>
    <td valign="top"><img src="$url" alt="[   ]" /></td>
    <td><a $add href="$file">$file $other</a></td>
    <td align="right">$date</td>
    <td align="right">$size</td>
    <td>&nbsp;</td>
</tr>"""
)



"""builder"""

import argparse
from os import makedirs, listdir
from os.path import getmtime, getsize, join, isdir, exists
from datetime import datetime

from .assets import FOLDER_ICO, HTML_TEMPLATE, ROW_TEMPLATE, UNKNOWN_ICO, CSS


def size(size_in_bytes):
    """convert"""
    size_kb = size_in_bytes / 1024.0
    size_mb = size_kb / 1024.0

    if size_mb >= 1:
        return f"{size_mb:.2f}M"
    elif size_kb >= 1:
        return f"{size_kb:.2f}K"
    return size_in_bytes


IMAGES_EXTS = ["png", "jpg", "jpeg", "tiff", "svg"]
VIEWABLE_EXTS = ["html", "htm", "pdf", "txt", "md", "csv", "json", "xml", "yaml", "yml"]
DEFAULT_ADDRESS = (
    'List generated by <a href="https://github.com/Its-Just-Nans/indexage">indexage</a>'
)


def create_html_index(path_to_folder, html_template, options):
    """create an index.html"""
    folder_list = sorted(listdir(path_to_folder))
    buffer_rows = ""
    current_template = f"{html_template}"
    correct_html = current_template.replace(
        "<title></title>", f"<title>Index of {path_to_folder}</title>"
    )
    folder_name = path_to_folder[2:]
    link_to_add = ""
    if options["link"]:
        link = f"{options['link']}{folder_name}"
        link_to_add = f' - <a href="{link}">{link}</a>'
    correct_html = correct_html.replace(
        "<h1></h1>",
        f"<h1>Index of {path_to_folder}{link_to_add}</h1>",
    )
    for one_element in folder_list:
        if one_element == "index.html" or one_element in options["exclude"]:
            continue
        path_to_element = join(path_to_folder, one_element)
        if path_to_element in options["exclude"]:
            continue
        is_element_dir = isdir(path_to_element)
        date = datetime.fromtimestamp(getmtime(path_to_element)).strftime(
            "%Y-%m-%d %H:%M:%S"
        )
        url = FOLDER_ICO if is_element_dir else UNKNOWN_ICO
        preview = options["preview"] and not is_element_dir
        add = 'class="thumbnail"' if preview else ""
        other = ""
        if preview:
            extension = one_element[one_element.rfind(".") + 1 :]
            if extension in IMAGES_EXTS:
                other = f'<span><img src="{one_element}"></span>'
            elif extension in VIEWABLE_EXTS:
                other = f'<span><iframe src="{one_element}"></iframe></span>'
        buffer_rows += ROW_TEMPLATE.substitute(
            file=one_element,
            date=date,
            size=size(getsize(path_to_element)),
            url=url,
            add=add,
            other=other,
        )
        if is_element_dir and one_element not in options["exclude"]:
            makedirs(options["output"], exist_ok=True)
            new_output = join(options["output"], one_element)
            new_options = options.copy()
            new_options["output"] = new_output
            if options["verbose"]:
                print(f"making {path_to_element}")
            create_html_index(path_to_element, html_template, new_options)
    file_data = correct_html.replace("<placeholder></placeholder>", buffer_rows)
    output_index = join(options["output"], "index.html")
    if not options["force"] and exists(output_index):
        raise FileExistsError(f"File {output_index} already exists, aborting")
    with open(output_index, "w", encoding="utf-8") as file:
        file.write(file_data)


def main():
    """main function"""
    parser = argparse.ArgumentParser(
        prog="indexage", description="Create apache2-like html index"
    )
    parser.add_argument("-o", "--output", type=str, help="Output folder", default=".")
    parser.add_argument(
        "-force",
        "--force",
        action=argparse.BooleanOptionalAction,
        type=bool,
        help="Force writing index.html",
        default=False,
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action=argparse.BooleanOptionalAction,
        type=bool,
        help="Add verbosity",
        default=False,
    )
    parser.add_argument(
        "-e", "--exclude", action="append", help="Exclude folder name", default=list()
    )
    parser.add_argument(
        "-a", "--address", type=str, help="Bottom address", default=DEFAULT_ADDRESS
    )
    parser.add_argument("-l", "--link", type=str, help="Add repo link", default="")
    parser.add_argument(
        "-p",
        "--preview",
        action=argparse.BooleanOptionalAction,
        type=bool,
        help="Add preview when hovering",
        default=False,
    )
    parser.add_argument("path", type=str, help="Paths to index")
    args = parser.parse_args()
    if args.verbose:
        print(args)
    options = {
        "exclude": args.exclude,
        "verbose": args.verbose,
        "link": args.link,
        "preview": args.preview,
        "force": args.force,
        "output": args.output.strip() == "" and "." or args.output,
    }
    template = HTML_TEMPLATE
    if options["preview"]:
        template = template.replace("<style></style>", f"<style>{CSS}</style>")
    template = template.replace(
        "<address></address>", f"<address>{args.address}</address>"
    )
    create_html_index(args.path, template, options)
